using System;
using System.IO;
using NUnit.Framework;

namespace Smdn.Formats.Thbgm {
  [TestFixture]
  public class ProductInfoTests {
    [Test]
    public void TestEmbeddedProductInfo()
    {
      Assert.IsNotNull(ProductInfo.Th06);
      Assert.IsNotNull(ProductInfo.Th07);
      Assert.IsNotNull(ProductInfo.Th08tr);
      Assert.IsNotNull(ProductInfo.Th08);
      Assert.IsNotNull(ProductInfo.Th09tr);
      Assert.IsNotNull(ProductInfo.Th09);
      Assert.IsNotNull(ProductInfo.Th095);
      Assert.IsNotNull(ProductInfo.Th10tr);
      Assert.IsNotNull(ProductInfo.Th10);
      Assert.IsNotNull(ProductInfo.Th10tr);
      Assert.IsNotNull(ProductInfo.Th10trWeb);
      Assert.IsNotNull(ProductInfo.Th11);
      Assert.IsNotNull(ProductInfo.Th11tr);
      Assert.IsNotNull(ProductInfo.Th11);
      Assert.IsNotNull(ProductInfo.Th12tr);
      Assert.IsNotNull(ProductInfo.Alcostg);

      Assert.AreEqual(17, ProductInfo.Th06.Tracks.Count,      "track count of Th06");
      Assert.AreEqual(20, ProductInfo.Th07.Tracks.Count,      "track count of Th07");
      Assert.AreEqual(7,  ProductInfo.Th08tr.Tracks.Count,    "track count of Th08tr");
      Assert.AreEqual(21, ProductInfo.Th08.Tracks.Count,      "track count of Th08");
      Assert.AreEqual(7,  ProductInfo.Th09tr.Tracks.Count,    "track count of Th09tr");
      Assert.AreEqual(19, ProductInfo.Th09.Tracks.Count,      "track count of Th09");
      Assert.AreEqual(6,  ProductInfo.Th095.Tracks.Count,     "track count of Th095");
      Assert.AreEqual(7,  ProductInfo.Th10tr.Tracks.Count,    "track count of Th10tr");
      Assert.AreEqual(8,  ProductInfo.Th10trWeb.Tracks.Count, "track count of Th10trWeb");
      Assert.AreEqual(18, ProductInfo.Th10.Tracks.Count,      "track count of Th10");
      Assert.AreEqual(8,  ProductInfo.Th11tr.Tracks.Count,    "track count of Th11tr");
      Assert.AreEqual(18, ProductInfo.Th11.Tracks.Count,      "track count of Th11");
      Assert.AreEqual(8,  ProductInfo.Th12tr.Tracks.Count,    "track count of Th12tr");
      Assert.AreEqual(18, ProductInfo.Th12.Tracks.Count,      "track count of Th12");

      foreach (var expected in new[] {
        new {Creator = ProductInfo.Th06.Creator,      Message = "creator of Th06"},
        new {Creator = ProductInfo.Th07.Creator,      Message = "creator of Th07"},
        new {Creator = ProductInfo.Th08.Creator,      Message = "creator of Th08"},
        new {Creator = ProductInfo.Th08tr.Creator,    Message = "creator of Th08tr"},
        new {Creator = ProductInfo.Th09.Creator,      Message = "creator of Th09"},
        new {Creator = ProductInfo.Th09tr.Creator,    Message = "creator of Th09tr"},
        new {Creator = ProductInfo.Th095.Creator,     Message = "creator of Th095"},
        new {Creator = ProductInfo.Th10.Creator,      Message = "creator of Th10"},
        new {Creator = ProductInfo.Th10tr.Creator,    Message = "creator of Th10tr"},
        new {Creator = ProductInfo.Th10trWeb.Creator, Message = "creator of Th10trWeb"},
        new {Creator = ProductInfo.Th11.Creator,      Message = "creator of Th11"},
        new {Creator = ProductInfo.Th11tr.Creator,    Message = "creator of Th11tr"},
        new {Creator = ProductInfo.Th12.Creator,      Message = "creator of Th12"},
        new {Creator = ProductInfo.Th12tr.Creator,    Message = "creator of Th12tr"},
      }) {
        Assert.AreEqual("上海アリス幻樂団", expected.Creator, expected.Message);
      }
    }

    [Test]
    public void TestGetUnifiedTitle()
    {
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢  ～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢  ～ Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢  ～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.DoubleSpace));
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢  ～ Perfect Cherry Blossom.", TitleUnificationOptions.SingleIdeographicSpace));

      Assert.AreEqual("東方妖々夢　\u301c Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　\u301c Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));
      Assert.AreEqual("東方妖々夢　\uff5e Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　\uff5e Perfect Cherry Blossom.", TitleUnificationOptions.NoUnify));

      Assert.AreEqual("東方妖々夢　〜 Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　～ Perfect Cherry Blossom.", TitleUnificationOptions.WaveDash));
      Assert.AreEqual("東方妖々夢　～ Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢　〜 Perfect Cherry Blossom.", TitleUnificationOptions.FullWidthTilde));

      Assert.AreEqual("東方妖々夢\u0020\u0020\u301c Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢\u3000\uff5e Perfect Cherry Blossom.", TitleUnificationOptions.DoubleSpace | TitleUnificationOptions.WaveDash));
      Assert.AreEqual("東方妖々夢\u3000\uff5e Perfect Cherry Blossom.", ProductInfo.GetUnifiedTitle("東方妖々夢\u0020\u0020\u301c Perfect Cherry Blossom.", TitleUnificationOptions.SingleIdeographicSpace | TitleUnificationOptions.FullWidthTilde));
    }
  }
}