﻿// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Drawing;
using System.Windows.Forms;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class CollectiveEffectEditDialog : Form {
    private class FadingOffsetDropDownList : FormattingDropDownList<int> {
      public FadingOffsetDropDownList()
      {
        Items.Add(-1);
        Items.Add(+1);

        StringFormatter = delegate(int val) {
          if (0 < val)
            return Texts.GetText("FadingOffset_AfterRepeatFinished");
          else
            return Texts.GetText("FadingOffset_BeforeRepeatFinished");
        };
      }
    }

    public EffectLengthBasis EffectLengthBasis {
      get { return effectLengthBasisDropDownList.SelectedItem; }
    }

    public TimeSpan FadingStart {
      get { return timeSpanUpDownFadingStart.Value; }
    }

    public TimeSpan TotalLength {
      get { return timeSpanUpDownTotalLength.Value; }
    }

    public int TimesToRepeat {
      get { return (int)numericUpDownTimesToRepeat.Value; }
    }

    public TimeSpan FadingOffset {
      get
      {
        if (0 < fadingOffsetDropDownList.SelectedItem)
          return timeSpanUpDownFadingOffset.Value;
        else
          return -timeSpanUpDownFadingOffset.Value;
      }
    }

    public TimeSpan FadingLength {
      get { return timeSpanUpDownFadingLength.Value; }
    }

    public FadingCurveType FadingCurveType {
      get { return fadingCurveTypeDropDownList.SelectedItem; }
    }

    public TimeSpan TrailingSilenceLength {
      get { return timeSpanUpDownTrailingSilenceLength.Value; }
    }

    public CollectiveEffectEditDialog()
    {
      InitializeComponent();

      timeSpanUpDownFadingStart.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingStart.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingStart.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingStart.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingStart.Increment = TimeSpan.FromSeconds(10.0);
      timeSpanUpDownFadingStart.Value = TimeSpan.FromMinutes(5.0); // 5 mins

      timeSpanUpDownFadingLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingLength.Increment = TimeSpan.FromSeconds(0.2);
      timeSpanUpDownFadingLength.Value = TimeSpan.FromSeconds(20.0); // 20 secs

      timeSpanUpDownFadingOffset.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingOffset.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingOffset.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingOffset.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingOffset.Increment = TimeSpan.FromSeconds(0.2);
      timeSpanUpDownFadingOffset.Value = TimeSpan.Zero;

      timeSpanUpDownTrailingSilenceLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownTrailingSilenceLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownTrailingSilenceLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownTrailingSilenceLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownTrailingSilenceLength.Increment = TimeSpan.FromSeconds(0.5);
      timeSpanUpDownTrailingSilenceLength.Value = TimeSpan.Zero; // 0 secs

      timeSpanUpDownTotalLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownTotalLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownTotalLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownTotalLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownTotalLength.Increment = TimeSpan.FromSeconds(10.0);
      timeSpanUpDownTotalLength.Value = TimeSpan.FromMinutes(5.0); // 5 mins

      numericUpDownTimesToRepeat.Minimum = 0;
      numericUpDownTimesToRepeat.Maximum = int.MaxValue;
      numericUpDownTimesToRepeat.Increment = 1m;
      numericUpDownTimesToRepeat.Value = 2m; // 2 loops

      panelEffectLengthBasisEdit.Left = 0;
      panelEffectLengthBasisEdit.Top = -timeSpanUpDownFadingStart.Top;

      timeSpanUpDownFadingOffset.Enabled = false;
      labelFadingOffset.Enabled = false;
      fadingOffsetDropDownList.Enabled = false;
      fadingOffsetDropDownList.SelectedIndex = 0;

      effectLengthBasisDropDownList.SelectedIndexChanged += delegate {
        var timesToRepeat = false;

        switch (effectLengthBasisDropDownList.SelectedItem) {
          case EffectLengthBasis.FadingStart:
            panelEffectLengthBasisEdit.Top = -timeSpanUpDownFadingStart.Top;
            break;
          case EffectLengthBasis.TotalLength:
            panelEffectLengthBasisEdit.Top = -timeSpanUpDownTotalLength.Top;
            break;
          case EffectLengthBasis.TimesToRepeat:
            panelEffectLengthBasisEdit.Top = -numericUpDownTimesToRepeat.Top;
            timesToRepeat = true;
            break;
        }

        timeSpanUpDownFadingOffset.Enabled = timesToRepeat;
        fadingOffsetDropDownList.Enabled = timesToRepeat;
        labelFadingOffset.Enabled = timesToRepeat;
      };

      effectLengthBasisDropDownList.SelectedIndex = 0;
    }

    private void buttonOK_Click(object sender, EventArgs e)
    {
      DialogResult = DialogResult.OK;

      Close();
    }
  }
}
