// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public partial class EffectEditor : UserControl {
    /*
     * properties
     */
    public Effect Effect {
      get { return effect; }
      set
      {
        if (effect == value)
          return;

        if (effect != null)
          effect.PropertyChanged -= Effect_PropertyChanged;

        effect = value;

        if (effect != null)
          effect.PropertyChanged += Effect_PropertyChanged;

        OnEffectChanged();
      }
    }

    /*
     * construction/destruction
     */
    public EffectEditor()
    {
      InitializeComponent();

      fadingCurveTypeDropDownList.SelectedIndexChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;

        updatingEffectProperty = true;
        effect.FadingCurveType = (sender as FadingCurveTypeDropDownList).SelectedItem;
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingStart.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingStart.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingStart.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingStart.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingStart.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingStart.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingStart = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownFadingLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownFadingLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownFadingLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownFadingLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownFadingLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownFadingLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.FadingLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };

      timeSpanUpDownTrailingSilenceLength.TimeSpanToStringConverter = Effect.TimeSpanToStringConverter;
      timeSpanUpDownTrailingSilenceLength.StringToTimeSpanConverter = Effect.StringToTimeSpanConverter;
      timeSpanUpDownTrailingSilenceLength.Minimum = TimeSpan.Zero;
      timeSpanUpDownTrailingSilenceLength.Maximum = Effect.TimeSpanUIMaxValue;
      timeSpanUpDownTrailingSilenceLength.Increment = new TimeSpan(0, 0, 0, 0, 200);
      timeSpanUpDownTrailingSilenceLength.ValueChanged += delegate(object sender, EventArgs e) {
        if (effect == null || updatingEffectProperty)
          return;
        updatingEffectProperty = true;
        effect.TrailingSilenceLength = (sender as TimeSpanUpDown).Value;
        updatingEffectProperty = false;
      };
    }

    private void OnEffectChanged()
    {
      this.Enabled = (effect != null);

      if (effect != null)
        UpdateValues();
    }

    private void UpdateValues()
    {
      updatingEffectProperty = true;

      for (var i = 0; i < fadingCurveTypeDropDownList.Items.Count; i++) {
        if (fadingCurveTypeDropDownList.Items[i] == effect.FadingCurveType)
          fadingCurveTypeDropDownList.SelectedIndex = i;
      }

      timeSpanUpDownFadingStart.Value = effect.FadingStart;
      timeSpanUpDownFadingLength.Value = effect.FadingLength;
      timeSpanUpDownTrailingSilenceLength.Value = effect.TrailingSilenceLength;

      UpdateTotalLength();

      updatingEffectProperty = false;
    }

    private void UpdateTotalLength()
    {
      labelTotalLength.Text = string.Format("{0:D}:{1:D2}:{2:D2}.{3:D3}", (int)effect.TotalLength.TotalHours, effect.TotalLength.Minutes, effect.TotalLength.Seconds, effect.TotalLength.Milliseconds);
    }

    private void Effect_PropertyChanged(object sender, PropertyChangedEventArgs e)
    {
      if (sender == effect) {
        if (updatingEffectProperty)
          UpdateTotalLength();
        else
          UpdateValues();
      }
    }

    private Effect effect = null;
    private bool updatingEffectProperty = false;
  }
}
