﻿// 
// Author:
//       smdn <smdn@mail.invisiblefulmoon.net>
// 
// Copyright (c) 2009 smdn
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Windows.Forms;

namespace Smdn.Windows.Forms.ThbgmPlayer {
  public class TrackListViewItem : ListViewItem {
    public Track Track {
      get; private set;
    }

    public TrackListViewItem(Track track)
    {
      if (track == null)
        throw new ArgumentNullException("track");

      this.Track = track;
      this.Track.Effect.PropertyChanged += Effect_PropertyChanged;

      UpdateSubItems();
    }

    private void UpdateSubItems()
    {
      SubItems.Clear();

      Text = Track.TrackInfo.TrackNumber.ToString();

      SubItems.Add(Track.TrackInfo.Title);
#if DEBUG
      SubItems.Add(Track.IsDirty.ToString());
#endif
      SubItems.Add(ToString(Track.Effect.TotalLength));
      SubItems.Add(Track.TrackInfo.CalculateRepeatedTimes(Track.Effect.TotalLength).ToString());
      SubItems.Add(ToString(Track.Effect.FadingStart));
      SubItems.Add(ToString(Track.Effect.FadingLength));
      SubItems.Add(Texts.GetText(Track.Effect.FadingCurveType));
      SubItems.Add(Track.Effect.TrailingSilenceLength.ToString());

      Checked = Track.Checked;
    }

    private string ToString(TimeSpan t)
    {
      return string.Format("{0}:{1:D2}.{2:D3}", (int)t.TotalMinutes, t.Seconds, t.Milliseconds);
    }

    private void Effect_PropertyChanged(object sender, EventArgs e)
    {
      if (sender == Track.Effect)
        UpdateSubItems();
    }
  }
}
