﻿using System;
using System.Collections.Generic;

using Smdn.Formats.Ini;

namespace Smdn.Applications.ThbgmExtractor {
  public class EncoderConfig {
    /*
     * class members
     */
    public static string ConfigFileName = @"Encoders.conf";
    public static EncoderConfig Empty = new EncoderConfig(ThbgmExtractor.GetText("EncoderConfig_Empty_Name"));

    public static IEnumerable<EncoderConfig> LoadFrom(string configFile)
    {
      var configs = new List<EncoderConfig>(new[] {Empty});

      try {
        var document = IniDocument.Load(configFile);

        foreach (var section in document.Sections) {
          try {
            if (!string.IsNullOrEmpty(section.Name))
              configs.Add(new EncoderConfig(section));
          }
          catch {
            // ignore invalid section
          }
        }
      }
      catch (System.IO.IOException) {
        // ignore I/O exceptions
      }

      return configs;
    }

    public static bool IsNullOrEmpty(EncoderConfig encoderConfig)
    {
      if (encoderConfig == null)
        return true;
      else if (encoderConfig == Empty)
        return true;
      else
        return false;
    }

    /*
     * instance members
     */
    public string Name {
      get; private set;
    }

    public string ExecutableFile {
      get; private set;
    }

    public string Arguments {
      get; private set;
    }

    public string OutputExtension {
      get; private set;
    }

    private EncoderConfig(string name)
    {
      this.Name = name;
    }

    private EncoderConfig(IniSection section)
    {
      this.Name = section.Name;
      this.ExecutableFile   = Environment.ExpandEnvironmentVariables(section["executable"]);
      this.Arguments        = section["arguments"];
      this.OutputExtension  = section["extension"];
    }

    public override string ToString()
    {
      return Name;
    }
  }
}
